@@ -57,7 +57,7 @@ class InvenTreeAPI(object):
 
         self.api_url = os.path.join(self.base_url, 'api/')
 
-        logger.info("Connecting to server: " + str(self.base_url))
+        logger.info(f"Connecting to server: {self.base_url}")
 
         self.username = kwargs.get('username', None)
         self.password = kwargs.get('password', None)
@@ -97,31 +97,28 @@ class InvenTreeAPI(object):
         logger.info("Checking InvenTree server connection...")
 
         try:
-            response = requests.get(self.api_url)
-        except requests.exceptions.ConnectionError:
-            logger.error("Server connection refused - check server address")
+            response = requests.get(self.api_url, timeout=2.5)
+        except requests.exceptions.ConnectionError as e:
+            logger.fatal("Server connection error:", type(e))
             return False
+        except Exception as e:
+            logger.fatal("Unhandled server error:", type(e))
+            # Re-throw the exception
+            raise e
 
-        if not response.status_code == 200:
-            logger.error("Error code from server: {code} - {detail}".format(
-                code=response.status_code,
-                detail=response.text
-            ))
-
-            return False
+        if response.status_code != 200:
+            raise requests.exceptions.RequestException(f"Error code from server: {response.status_code} - {response.text}")
 
         # Record server details
         self.server_details = json.loads(response.text)
 
-        logger.info("InvenTree server details: " + str(response.text))
+        logger.info(f"InvenTree server details: {response.text}")
 
         # The details provided by the server should include some specific data:
         server_name = str(self.server_details.get('server', ''))
 
         if not server_name.lower() == 'inventree':
-            logger.warning("Server returned strange response (expected 'InvenTree', found '{name}')".format(
-                name=server_name
-            ))
+            logger.warning(f"Server returned strange response (expected 'InvenTree', found '{server_name}')")
 
         api_version = self.server_details.get('apiVersion', '1')
 
@@ -151,21 +148,16 @@ class InvenTreeAPI(object):
         data = json.loads(reply.text)
 
         if not reply.status_code == 200:
-            logger.error("Error requesting token: {code} - {detail}".format(
-                code=reply.status_code,
-                detail=str(reply.text)
-            ))
+            logger.error(f"Error requesting token: {reply.status_code} - {reply.text}")
             return None
 
         if 'token' not in data.keys():
-            logger.error("Token not returned by server: {detail}".format(
-                detail=str(reply.text)
-            ))
+            logger.error(f"Token not returned by server: {reply.text}")
             return None
 
         self.token = json.loads(reply.text)['token']
 
-        logger.info("Authentication token: " + self.token)
+        logger.info(f"Authentication token: {self.token}")
 
         return self.token
 
@@ -206,25 +198,26 @@ class InvenTreeAPI(object):
         }
 
         if method.upper() not in methods.keys():
-            logger.error("Unknown request method '{m}'".format(m=method))
+            logger.error(f"Unknown request method '{method}'")
             return None
 
         method = method.upper()
 
         if self.use_token_auth and self.token:
-            headers['AUTHORIZATION'] = 'Token {t}'.format(t=self.token)
+            headers['AUTHORIZATION'] = f'Token {self.token}'
             auth = None
         else:
             auth = self.auth
 
         logger.debug("Sending Request:")
-        logger.debug(" - URL:", method, api_url)
-        logger.debug(" - auth:", auth)
-        logger.debug(" - params:", params)
-        logger.debug(" - headers:", headers)
-        logger.debug(" - json:", json)
-        logger.debug(" - files:", files)
+        logger.debug(f" - URL: {method} {api_url}")
+        logger.debug(f" - auth: {auth}")
+        logger.debug(f" - params: {params}")
+        logger.debug(f" - headers: {headers}")
+        logger.debug(f" - json: {json}")
+        logger.debug(f" - files: {files}")
 
+        # Send request to server!
         try:
             response = methods[method](
                 api_url,
@@ -232,26 +225,45 @@ class InvenTreeAPI(object):
                 params=params,
                 headers=headers,
                 json=json,
-                files=files
+                files=files,
+                timeout=10,
             )
-
-        except requests.exceptions.ConnectionError:
-            logger.error("Connection refused - '{url}'".format(url=api_url))
-            return None
+        except Exception as e:
+            # Re-thrown any caught errors, and add a message to the log
+            logger.critical(f"Error at api.request - {method} @ {api_url}")
+            raise e
 
         if response is None:
-            logger.error("Null response - {method} '{url}'".format(method=method, url=api_url))
+            logger.error(f"Null response - {method} '{api_url}'")
             return None
 
-        logger.info("Request: {method} {url} - {response}".format(method=method, url=api_url, response=response.status_code))
+        logger.info(f"Request: {method} {api_url} - {response.status_code}")
 
         # Detect invalid response codes
         # Anything 300+ is 'bad'
         if response.status_code >= 300:
-            logger.warning("Bad response ({code}) - {method} '{url}' - {detail}".format(
-                code=response.status_code, method=method, url=api_url,
-                detail=str(response.text)
-            ))
+
+            detail = {
+                'detail': 'Error occurred during API request',
+                'url': api_url,
+                'method': method,
+                'status_code': response.status_code,
+                'body': response.text,
+            }
+
+            if headers:
+                detail['headers'] = headers
+            
+            if params:
+                detail['params'] = params
+            
+            if files:
+                detail['files'] = files
+            
+            if json:
+                detail['data'] = json
+
+            raise requests.exceptions.HTTPError(detail)
 
         # A delete request won't return JSON formatted data (ignore further checks)
         if method == 'DELETE':
@@ -259,9 +271,11 @@ class InvenTreeAPI(object):
 
         ctype = response.headers.get('content-type')
 
+        # An API request must respond in JSON format
         if not ctype == 'application/json':
-            logger.error("'Response content-type is not JSON - '{url}' - '{f}'".format(url=api_url, f=ctype))
-            return None
+            raise requests.exceptions.InvalidJSONError(
+                f"'Response content-type is not JSON - '{api_url}' - '{ctype}'"
+            )
 
         return response
 
@@ -280,7 +294,7 @@ class InvenTreeAPI(object):
         if response.status_code not in [204]:
             logger.error(f"DELETE request failed at '{url}' - {response.status_code}")
 
-        logger.debug(response.status_code, response.text)
+        logger.debug(f"DELETE request at '{url}' returned: {response.status_code} {response.text}")
 
         return response
 
@@ -298,7 +312,7 @@ class InvenTreeAPI(object):
         headers = kwargs.get('headers', {})
 
         if self.use_token_auth and self.token:
-            headers['AUTHORIZATION'] = 'Token {t}'.format(t=self.token)
+            headers['AUTHORIZATION'] = f'Token {self.token}'
             auth = None
         else:
             auth = self.auth
@@ -309,14 +323,14 @@ class InvenTreeAPI(object):
             return None
 
         if response.status_code not in [200, 201]:
-            logger.error("POST request failed at '{url}' - {status}".format(url=url, status=response.status_code))
-            logger.error(response.text)
+            logger.error(f"POST request failed at '{url}' - {response.status_code}")
+            logger.error(f"{response.text}")
             return None
         
         try:
             data = json.loads(response.text)
         except json.decoder.JSONDecodeError:
-            logger.error("Error decoding JSON response - '{url}'".format(url=url))
+            logger.error(f"Error decoding JSON response - '{url}'")
             return None
 
         return data
@@ -422,15 +436,18 @@ class InvenTreeAPI(object):
         try:
             data = json.loads(response.text)
         except json.decoder.JSONDecodeError:
-            logger.error("Error decoding JSON response - '{url}'".format(url=url))
+            logger.error(f"Error decoding JSON response - '{url}'")
             return None
 
         return data
 
-    def downloadFile(self, url, destination):
+    def downloadFile(self, url, destination, overwrite=False):
         """
         Download a file from the InvenTree server.
 
+        Args:
+            destination: Filename (string)
+
         - If the "destination" is a directory, use the filename of the remote URL
         """
 
@@ -451,6 +468,9 @@ class InvenTreeAPI(object):
 
         destination = os.path.abspath(destination)
 
+        if os.path.exists(destination) and not overwrite:
+            raise FileExistsError(f"Destination file '{destination}' already exists")
+
         if self.token:
             headers = {
                 'AUTHORIZATION': f"Token {self.token}"
@@ -460,27 +480,31 @@ class InvenTreeAPI(object):
             headers = {}
             auth = self.auth
 
-        with requests.get(url, stream=True, auth=auth, headers=headers) as request:
+        with requests.get(url, stream=True, auth=auth, headers=headers) as response:
 
-            if not request.status_code == 200:
-                logger.error(
-                    f"Error downloading file '{url}': Server returned status {request.status_code}"
-                )
+            # Error code
+            if response.status_code >= 300:
+                detail = {
+                    'detail': 'Error occurred during file download',
+                    'url': url,
+                    'status_code': response.status_code,
+                    'body': response.text
+                }
 
-                return False
+                if headers:
+                    detail['headers'] = headers
+                
+                raise requests.exceptions.HTTPError(detail)
 
-            headers = request.headers
+            headers = response.headers
 
             if 'text/html' in headers['Content-Type']:
-                logger.error(
-                    f"Error downloading file '{url}': Server return invalid response (text/html)"
-                )
-
+                logger.error(f"Error downloading file '{url}': Server return invalid response (text/html)")
                 return False
 
             with open(destination, 'wb') as f:
 
-                for chunk in request.iter_content(chunk_size=16 * 1024):
+                for chunk in response.iter_content(chunk_size=16 * 1024):
                     f.write(chunk)
 
         logger.info(f"Downloaded '{url}' to '{destination}'")
